# == Schema Information
# Schema version: 20090612184651
#
# Table name: championships
#
#  id                      :integer         not null, primary key
#  name                    :string(255)     not null
#  state                   :string(255)     default("open")
#  rules                   :text
#  created_at              :datetime
#  updated_at              :datetime
#  championship_manager_id :integer
#

class Championship < ActiveRecord::Base
  
  hobo_model # Don't put anything above this
  
  fields do
    name :string, :null => false
    state enum_string(:open, :closed), :default => "open"
    rules :html
    timestamps
  end
  
  belongs_to :championship_manager, :class_name => "User", :creator => true
  has_many :games, :dependent => :destroy
  has_many :memberships, :class_name => "ChampionshipMembership", :order => "score DESC", :dependent => :destroy
  has_many :members, :through => :memberships, :source => :user
  
  validates_presence_of :name
  validates_presence_of :state
  validates_presence_of :championship_manager
  validates_uniqueness_of :name
  
  
  def owner
    championship_manager
  end
  
  def after_create
    memberships.create!(:user => acting_user)
  end
  
  def compute_stats_for_user(user)
    championship_membership = memberships.find_by_user_id(user)
    championship_membership.score = 0
    championship_membership.goals = 0
    games.each { |game|
      game_membership = game.memberships.find_by_user_id(user)
      championship_membership.score += game_membership.score if game_membership
      championship_membership.goals += game_membership.goals if game_membership
    }
    championship_membership.save!
  end
  
  # --- Hobo Permissions --- #
  
  def create_permitted?
    return unless acting_user.signed_up? && championship_manager_is?(acting_user) 
    
    state == "open"
  end
  
  def update_permitted?
    acting_user.administrator? || state_was == 'open' && (championship_manager_is?(acting_user) &&  !championship_manager_changed?)
  end
  
  def destroy_permitted?
    acting_user.administrator?
  end
  
  def view_permitted?(attribute)
    return true
  end
end
